library(statmod)

fit.rcb.mixeff.mle <- function(diaper, extra.tx, phi.init = NULL, nGQ = 20)
{
	X <- model.matrix(~X, data = diaper)	
	d <- ncol(X)
	
	Data <- list(y = diaper$t, m = diaper$m, X = X, n = max(diaper$SubjId))

	if (is.null(phi.init))
		phi.init <- c(rep(0,d+2))
	
	theta.tx <- function(phi)
	{
		list(Beta = phi[1:d], rho = plogis(phi[1+d]), sigma.b = exp(phi[2+d]))
	}

	loglik <- function(phi, Data)
	{
		theta <- theta.tx(phi)
		quad.out <- gauss.quad.prob(n = nGQ, dist = "normal", mu = 0, sigma = theta$sigma.b)
		u <- quad.out$nodes
		w <- quad.out$weights

		ll <- numeric(Data$n)
		eta.fixed <- Data$X %*% theta$Beta

		# Each subject has two observations, which are consecutive in the dataset
		# These correspond to the two periods of the experiment
		for (j in 1:Data$n)
		{
			idx.p1 <- 2*j - 1
			idx.p2 <- 2*j
			mu.p1 <- plogis(eta.fixed[idx.p1] + u)
			mu.p2 <- plogis(eta.fixed[idx.p2] + u)
			GQ.points.p1 <- d.rcb(Data$y[idx.p1], Pi = mu.p1, theta$rho, Data$m[idx.p1])
			GQ.points.p2 <- d.rcb(Data$y[idx.p2], Pi = mu.p2, theta$rho, Data$m[idx.p2])
			GQ.points <- GQ.points.p1 * GQ.points.p2

			ll[j] <- log(sum(w * GQ.points))
		}

		return(sum(ll))
	}

	start <- Sys.time()
	fit.out <- fit.mle(phi.init, loglik, theta.tx, extra.tx = extra.tx, Data = Data)

	fit.out$description <- paste0(
		sprintf("y[s,i,j] ~ind~ RCB(m[s,i,j], Pi[s,i,j], rho),\n"),
		sprintf("logit(Pi[s,i,j]) = x[s,i,j]^T Beta + u[j]\n"),
		sprintf("u[s,j] ~iid~ N(0, sigma.b^2)\n"),
		sprintf("Likelihood computed by Gaussian Quadrature with nGQ = %d\n", nGQ),
		sprintf("Elapsed time: %0.02f sec", as.numeric(Sys.time() - start, units = "secs"))
	)
	return(fit.out)
}

