set.seed(1234)

# --------- Parametric bootstrap for GOF statistic ---------
# Uses MLE (Pi.hat, rho.hat) computed from the observed data
B <- 200
n <- length(y); d1 <- nrow(X); d2 <- nrow(Z)
gof.out <- gof.bb.xz.out
X.boot <- numeric(B)

for (b in 1:B)
{
	printf("Starting bootstrap rep %d\n", b)
	y.boot <- r.beta.binom(n, Pi.hat, rho.hat, m)
	fit.boot <- fit.bb.xz.mle(y.boot, m, X, Z, phi.init = c(Beta.hat, Gamma.hat))

	Beta.hat.boot <- fit.boot$theta.hat$Beta
	Gamma.hat.boot <- fit.boot$theta.hat$Gamma
	Pi.hat.boot <- plogis(X %*% Beta.hat.boot)
	rho.hat.boot <- plogis(Z %*% Gamma.hat.boot)

	f.boot <- function(x, i) { d.beta.binom(x, Pi.hat.boot[i], rho.hat.boot[i], m[i]) }
	gof.out.boot <- gof.binomial(y.boot, m, f.boot, gof.breaks, qq = ncol(X) + ncol(Z))
	X.boot[b] <- gof.out.boot$X
}

p.value.boot <- mean(X.boot > gof.out$X)
printf("Finished bootstrap. p.value.boot = %g\n", p.value.boot)

# --------- Plot the bootstrap dist'n of the GOF statistic ---------
pdf("gof-bbreg-boot-ecdf.pdf", width = 5, height = 5)
plot(ecdf(X.boot))
curve(pchisq(x, df = gof.out$df.low), add = TRUE, col = "red", lwd = 2)
curve(pchisq(x, df = gof.out$df.high), add = TRUE, col = "green", lwd = 2)
mynames <- c("ECDF of X.boot",
	sprintf("chisq df = %d", gof.out$df.low),
	sprintf("chisq df = %d", gof.out$df.high))
mycol <- c("black", "red", "green")
legend("bottomright", mynames, col = mycol, lwd = 2)
dev.off()
