source("gof.R")

# --------- Generate the data ---------
n <- 100
m <- rpois(n, lambda = 20)
y <- r.rcb(n, Pi = 1/4, rho = 0.2, m)

# --------- Fit the MLE for the observed data ---------
extra.tx <- function(theta) { }
fit.out <- fit.rcb.mle(y, m, extra.tx)
Pi.hat <- fit.out$estimates["Pi",1]
rho.hat <- fit.out$estimates["rho",1]

# --------- Compute GOF for the observed data ---------
gof.breaks <- c(0, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 1)
f <- function(x,i) { d.rcb(x, Pi = Pi.hat, rho = rho.hat, m = m[i]) }
gof.out <- gof.binomial(y, m, f, gof.breaks, qq = 2)

# --------- Parametric bootstrap for GOF statistic ---------
B <- 500
X.boot <- numeric(B)

for (b in 1:B)
{
	printf("Starting bootstrap rep %d\n", b)
	y.boot <- r.rcb(n, Pi.hat, rho.hat, m)
			
	fit.out.boot <- fit.rcb.mle(y.boot, m, extra.tx)
	Pi.hat.boot <- fit.out.boot$estimates["Pi",1]
	rho.hat.boot <- fit.out.boot$estimates["rho",1]
				
	f.boot <- function(x,i)
	{
		d.rcb(x, Pi = Pi.hat.boot, rho = rho.hat.boot, m = m[i])
	}
	gof.out.boot <- gof.binomial(y.boot, m, f.boot, gof.breaks, qq = 2)
	X.boot[b] <- gof.out.boot$X
}
	
p.value.boot <- mean(X.boot > gof.out$X)

# --------- Plot the bootstrap dist'n of the GOF statistic ---------
plot(ecdf(X.boot))
curve(pchisq(x, df = gof.out$df.low), add = TRUE, col = "red", lwd = 2)
curve(pchisq(x, df = gof.out$df.high), add = TRUE, col = "green", lwd = 2)
mynames <- c("ECDF of X.boot",
	sprintf("chisq df = %d", gof.out$df.low),
	sprintf("chisq df = %d", gof.out$df.high))
mycol <- c("black", "red", "green")
legend("bottomright", mynames, col = mycol, lwd = 2)
